/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2013  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "GarbageCollector.h"
#include "ArgumentData.h"

using namespace Scratch;


namespace BromScript{
	GarbageCollector::GarbageCollector()
	{
		this->Buffer = null;
		this->BufferSize = 0;
		this->AllocateMoreSpace();
	}

	GarbageCollector::~GarbageCollector()
	{
		for (int i = 0; i < this->BufferSize; i++){
			if (this->Buffer[i] != null){
				for (int i2 = i + 1; i2 < this->BufferSize; i2++){
					if (this->Buffer[i] == this->Buffer[i2]){
						this->Buffer[i2] = null;
					}
				}

				Variable* var = this->Buffer[i];

				delete this->Buffer[i];
			}
		}

		free(this->Buffer);
	}

	BS_FUNCTION(GarbageCollector::RunWrapper){
		return Converter::ToVariable(args->BromScript->GC.Run());
	}

	int GarbageCollector::Run(){
		int cleaned = 0;

		int curc = 1337;
		while(curc > 0){
			curc = 0;
			for (int i = 0; i < this->BufferSize; i++){
				if (this->Buffer[i] != null && this->Buffer[i]->Using == 0){
					curc++;

					for (int i2 = i + 1; i2 < this->BufferSize; i2++){
						if (this->Buffer[i] == this->Buffer[i2]){
							curc++;
							this->Buffer[i2] = null;
						}
					}

					delete this->Buffer[i];
					this->Buffer[i] = null;
				}
			}

			cleaned += curc;
		}

		return cleaned * sizeof(Variable);
	}

	Variable* GarbageCollector::RegisterVariable(){
		Variable* var = new Variable();
		var->RegisteredInGC = true;

		for (int i = 0; i < this->BufferSize; i++){
			if (this->Buffer[i] == null){
				this->Buffer[i] = var;
				return var;
			}
		}

		this->AllocateMoreSpace();
		this->Buffer[this->BufferSize - 512] = var;
		return var;
	}

	Variable* GarbageCollector::RegisterVariable(Variable* var){
		if (var->RegisteredInGC)
			return var;

		var->RegisteredInGC = true;

		for (int i = 0; i < this->BufferSize; i++){
			if (this->Buffer[i] == null){
				this->Buffer[i] = var;
				return var;
			}
		}

		this->AllocateMoreSpace();
		this->Buffer[this->BufferSize - 512] = var;

		return var;
	}

	void GarbageCollector::AllocateMoreSpace(){
		this->BufferSize += 512;
		Variable** newbuff = (Variable**)malloc(sizeof(Variable*) * this->BufferSize);

		for (int i = 0; i < this->BufferSize; i++)
			newbuff[i] = null;

		if(this->Buffer != null) {
			memcpy(newbuff, this->Buffer, sizeof(Variable*) * (this->BufferSize - 512));
			free(this->Buffer);
		}

		this->Buffer = newbuff;
	}
}