/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2013  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Table.h"

using namespace Scratch;

namespace BromScript{
	extern int _tcount = 0;

	int Table::GetCount(){
		return _tcount;
	}

	Table::Table(Instance* bs){
		_tcount++;
		this->Count = 0;

		this->BromScript = bs;
		this->BufferSize = 0;
		this->Keys = null;
		this->Values = null;
		this->AllocateMoreSpace();
	}

	Table::~Table(void){
		_tcount--;

		this->Clear();
		free(this->Keys);
		free(this->Values);
	}

	void Table::Clear(){
		for (int i = 0; i < this->BufferSize; i++){
			if(this->Keys[i] != null){
				delete this->Keys[i];
			}
		}

		for (int i = 0; i < this->BufferSize; i++){
			if(this->Values[i] != null){
				this->Values[i]->Using--;
			}
		}
		
		free(this->Keys);
		free(this->Values);

		this->Count = 0;
		this->Keys = null;
		this->Values = null;
		this->BufferSize = 0;
		this->AllocateMoreSpace();
	}

	Variable* Table::GetByIndex(int i){
		if (i >= this->BufferSize || this->Values[i] == null)
			return this->BromScript->GC.RegisterVariable();

		return this->Values[i];
	}

	CString Table::GetKeyByIndex(int i){
		if (i >= this->BufferSize || this->Keys[i] == null)
			return "";

		return *this->Keys[i];
	}

	void Table::Set(CString key, Variable* var){
		this->BromScript->GC.RegisterVariable(var);

		var->Using++;

		if (this->HasKey(key)){
			for (int i = 0; i < this->BufferSize; i++){
				if(this->Keys[i] != null && *this->Keys[i] == key){
					if (this->Values[i]->ReadOnly){
						this->BromScript->Error(CString("Cannot override '") + key + "', it's read only!");
						return;
					}

					this->Values[i]->Using--;
					this->Values[i] = var;
					return;
				}
			}
		}else{
			this->Count++;

			for (int i = 0; i < this->BufferSize; i++){
				if (this->Values[i] == null){
					this->Keys[i] = new CString(key);
					this->Values[i] = var;
					return;
				}
			}
		}

		this->Count++;
		this->AllocateMoreSpace();

		this->Keys[this->BufferSize - 128] = new CString(key);
		this->Values[this->BufferSize - 128] = var;
	}

	void Table::Remove(CString key){
		for (int i = 0; i < this->BufferSize; i++){
			if(this->Keys[i] != null && *this->Keys[i] == key){
				this->Values[i]->Using--;
				this->Values[i] = null;

				delete this->Keys[i];
				this->Keys[i] = null;
				return;
			}
		}
	}

	bool Table::HasKey(CString key){
		for (int i = 0; i < this->BufferSize; i++){
			if (this->Keys[i] != null && *this->Keys[i] == key){
				return true;
			}
		}

		return false;
	}

	Variable* Table::Get(CString key){
		for (int i = 0; i < this->BufferSize; i++){
			if (this->Keys[i] != null && *this->Keys[i] == key){
				return this->Values[i];
			}
		}

		return this->BromScript->GC.RegisterVariable();
	}

	void Table::AllocateMoreSpace(){
		this->BufferSize += 128;
		Variable** newbuffv = (Variable**)malloc(sizeof(Variable*) * this->BufferSize);
		CString** newbuffk = (CString**)malloc(sizeof(CString*) * this->BufferSize);

		for (int i = 0; i < this->BufferSize; i++){
			newbuffv[i] = null;
			newbuffk[i] = null;
		}
		
		if(this->Values != null) {
			memcpy(newbuffv, this->Values, sizeof(Variable*) * (this->BufferSize - 128));
			free(this->Values);
		}

		if(this->Keys != null) {
			memcpy(newbuffk, this->Keys, sizeof(Variable*) * (this->BufferSize - 128));
			free(this->Keys);
		}
		
		this->Values = newbuffv;
		this->Keys = newbuffk;
	}

	Variable* Table::operator[](CString key){
		return this->Get(key);
	}
}